import{d as m,u as _,e as i,h as l,i as h,j as k,k as d,l as b,m as v,b as x,n as o}from"./Bj8SChEI.js";function j(r=!1){const t=m,e=t.l.u;if(!e)return;let f=()=>b(t.s);if(r){let a=0,s={};const g=v(()=>{let u=!1;const c=t.s;for(const n in c)c[n]!==s[n]&&(s[n]=c[n],u=!0);return u&&a++,a});f=()=>d(g)}e.b.length&&_(()=>{p(t,f),l(e.b)}),i(()=>{const a=h(()=>e.m.map(k));return()=>{for(const s of a)typeof s=="function"&&s()}}),e.a.length&&i(()=>{p(t,f),l(e.a)})}function p(r,t){if(r.l.s)for(const e of r.l.s)d(e);t()}const $={get data(){return o.data},get error(){return o.error},get params(){return o.params},get status(){return o.status}};x.updated.check;export{j as i,$ as p};
