const a={home:()=>"/",dataset:{sample:t=>`/samples/${t}`,sampleWithAnnotation:(t,s)=>`/samples/${t}?annotationId=${s}`,samples:t=>`/datasets/${t}/samples`,annotations:t=>`/datasets/${t}/annotations`}},o={toHome:()=>a.home(),toSamples:t=>a.dataset.samples(t),toAnnotations:t=>a.dataset.annotations(t),toSample:t=>a.dataset.sample(t),toSampleWithAnnotation:(t,s)=>a.dataset.sampleWithAnnotation(t,s)};export{o as r};
