import{t as h,a as l,s as e}from"../chunks/Bd4FQpE7.js";import{p as v,f as u,t as g,a as x,c as p,r as o,s as _}from"../chunks/Bj8SChEI.js";import{i as d}from"../chunks/4LrpTdiC.js";import{p as i}from"../chunks/DZz--ybP.js";var b=h("<h1> </h1> <p> </p>",1);function w(m,f){v(f,!1),d();var a=b(),t=u(a),n=p(t,!0);o(t);var r=_(t,2),c=p(r,!0);o(r),g(()=>{var s;e(n,i.status),e(c,(s=i.error)==null?void 0:s.message)}),l(m,a),x()}export{w as component};
